/****************************************************************
   Author: Minh B. Do (Arizona State University)
*****************************************************************/
package myplanner.basic_ds;

/**
 * GMySet: Class to store the grounded "Set" structure, which is an
 * action effect related to changing the value of a continuous function.
 * The "Set" contains of 3 parts, the function in the left hand side, the
 * assigment indication (increase/decrease etc.) and the GMathForm structure
 * as the right hand side */
public class GMySet {
    int  leftSide; // Ground func ID of left side
    int assign; // "=" : 0; "-=" : 1; "+=" : 2; "*=" : 3; "/=" : 4;
    GMathForm rightSide;

    public GMySet() {
	leftSide = 0;
	assign = 0;
	rightSide = new GMathForm();
    }

    public GMySet(int left, int assg, GMathForm right) {
	leftSide = left;
	assign = assg;
	rightSide = new GMathForm(right);
    }

    /** Set the "head" of the assigment to a specific ground function's ID */
    public void setLeftSide(int l) {
	leftSide = l;
    }

    public int getLeftSide( ) {
	return leftSide;
    }

    /** Set the assignment (0: "="; "-=" : 1; "+=" : 2; "*=" : 3; "/=" : 4;) */
    public void setAssign(int assg) {
	assign = assg;
    }

    public int getAssign() {
	return assign;
    }

    /** Set the "tail" (right side) of this assignment formula to a specific GMathForm */
    public void setRightSide(GMathForm m) {
	rightSide = new GMathForm(m);
    }

    public GMathForm getRightSide() {
	return new GMathForm(rightSide);
    }

    public String toString() {
	String s = "(" + assign + " ";

	s += leftSide + " " +
	    rightSide.toString() + ")";

	return s;
    }
}
