/****************************************************************
    Author: Minh B. Do - Arizona State University
*****************************************************************/
package myplanner.basic_ds;

/**
 *  GTest: Class to store the grounded Test structure
 *  which is used to check the metric resource (pre)condition
 *  represented by a comparison on some continous function. Like
 *  "Set", "Test" consists of 3 component, a ground function on
 *  the left side, a comparator (e.g <, >, =) and the GMathForm
 *  represent a value on the right side.
 */
public class GTest {
    int  leftSide; // Ground Function ID of the left side
    // "==" : 0; "<" : 1; "<=" : 2; ">" : 3; ">=" : 4;
    int  comparator;
    GMathForm rightSide;

    public GTest() {
	leftSide = 0;
	comparator = 0;
	rightSide = new GMathForm();
    }

    public GTest(int left, int c, GMathForm right) {
	leftSide = left;
	comparator = c;
	rightSide = right;
    }

    /** Set the left side, which is an Integer represent a function's ID */
    public void setLeftSide(int l) {
	leftSide = l;
    }

    public int getLeftSide( ) {
	return leftSide;
    }

    /** Set the Comparator: "==" : 0; "<" : 1; "<=" : 2; ">" : 3; ">=" : 4; */
    public void setComparator(int i) {
	comparator = i;
    }

    public int getComparator() {
	return comparator;
    }

    /** Set the right hand side, which is a GMathForm represent one value */
    public void setRightSide(GMathForm m) {
	rightSide = new GMathForm(m);
    }

    public GMathForm getRightSide() {
	return rightSide;
    }

    /** Function to do short printing */
    public String toString() {
	String s = "(";

	s += comparator + " " + leftSide + " " +
	    rightSide.toString() + ")";

	return s;
    }
}
