/********************************************************************
    Author: Minh B. Do -- Arizona State University
*******************************************************************/
package myplanner.complex_ds;
import myplanner.basic_ds.*;

import java.util.*;
import java.io.*;

/**
 *  GState: Store the "State" information. The state basically containing
 *  two part. A list of predicates that are true and their achivement time
 *  (stored in GPredDB), and a set of values for all the continous functions
 *  (stored in GMResDB). The state is part of the StatePair structure.
 *  Refer to the documents of StatePair, GPredDB, and GMResDB for details.
 */
public class GState {
    GPredDB predDB;
    GMResDB mresDB;

    public GState(int numFunc) {
	predDB = new GPredDB();
	mresDB = new GMResDB(numFunc);
    }

    public GState(GState s) {
	predDB = s.getPredDB();
	mresDB = s.getMResDB();
    }

    public void setPredDB(GPredDB p) {
	predDB = new GPredDB(p);
    }

    public GPredDB getPredDB() {
	return new GPredDB(predDB);
    }

    public void setMResDB(GMResDB m) {
	mresDB = new GMResDB(m);
    }

    public GMResDB getMResDB() {
	return new GMResDB(mresDB);
    }

    /** Check if an action is applicable to this state */
    public boolean applicable(GAction a) {
	if (predDB.applicable(a) && mresDB.applicable(a))
	    return true;
	else
	    return false;
    }

    /** Update a state (both PredDB and MResDB) upon applying an action */
    public void update(GAction a, float time) {
	predDB.update(a, time);
	mresDB.update(a, true);
    }

    /** Update a state according to the set of events */
    public void update(ArrayList eventList) {
	predDB.update(eventList);
    }
}
