/**********************************************************************
   Author: Minh B. Do (Arizona State University - binhminh@asu.edu)
**********************************************************************/
package myplanner.complex_ds;
import myplanner.basic_ds.*;

import java.util.*;

/**
 *  StateManager: StateManager has three elements: initial state,
 *  goal state, and a sorted array of intermediate states (StateInfo class)
 *  that are reachable from the initial state. The states in the queue
 *  are sorted according to their distance values (h+g).
 */
public class StateManager {
    /* For grounded InitState and GoalState */
    GState gInitState;
    GState gGoalState;

    ArrayList stateQueue; // Set of all GStateInfo explored. stateSet(0) = InitState
    // Sorted according to the distane to the goal-state
    int STATE_QUEUE_SIZE = 50000; // Limit on the number of states in queue

    public StateManager(int numFunc) {
	gInitState = new GState(numFunc);
	gGoalState = new GState(numFunc);

	stateQueue = new ArrayList();
    }

    /** Set the initial predicate manager in the initial state */
    public void setInitPredDB(ArrayList predIDs) {
	gInitState.setPredDB(new GPredDB(predIDs, 0));
    }

    /** Set the initial resource manager in the initial state */
    public void setInitMResDB(GMResDB mres) {
	gInitState.setMResDB(new GMResDB(mres));
    }

    /** Set the goal predicate manager */
    public void setGoalPredDB(GPredDB aPredDB) {
	gGoalState.setPredDB(aPredDB);
    }

    public GState getInitState() {
	return new GState(gInitState);
    }

    public GState getGoalState() {
	return new GState(gGoalState);
    }

    /** Add a new (explored) state to the search queue */
    public void addStateInfo(StateInfo s) {
	int stpSize = stateQueue.size();
	// Check the limit
	if( stpSize > STATE_QUEUE_SIZE ) {
	    stateQueue.remove(--stpSize);
	}

	for(int i = 0; i < stpSize; i++)
	    if( ((StateInfo) stateQueue.get(i)).getDistance() >= s.getDistance() ) {
		stateQueue.add(i,s);
		return;
	    }

	stateQueue.add(s);
    }

    /** Get the first (shortest distance) StateInfo in the queue to explore */
    public StateInfo getStateInfo() {
	if(stateQueue.size() == 0)
	    return null;

	return (StateInfo) stateQueue.remove(0);
    }

    /** Reset the queue to contains only the initial state */
    public void resetStateQueue(StateInfo initState) {
	stateQueue.clear();
	stateQueue.add(initState);
    }

    public void clearStateQueue() {
	stateQueue.clear();
    }
}
