/****************************************************************
     Author: Daniel Bryce - Arizona State University
****************************************************************/
package myplanner.gui;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import myplanner.utils.GreedyPost;
import myplanner.parsing.Grounding;
import myplanner.utils.OrderRelation;
import myplanner.utils.Ordering;
import java.net.*;


public class ActionWindow extends JPanel implements ListSelectionListener{
     //options
    boolean other = false;
    boolean removeRedundant = true;
    int lastSelected = -1;
    
    //plan info
    int plansize;
    GreedyPost greedyPost;
    Grounding grounding;
    ArrayList actSigList;
    ArrayList plan;
    float[] times;
    float[] durs;
    GanttChart chart;
    Ordering[] order;
    String output;
    
    //Components
    JEditorPane planDetail;
    JEditorPane actionDetail;
    JTabbedPane tabPane;
    JPanel actionPanel;
    JScrollPane planDetailPanel;
    Vector cells;
    JList actionList;
    JScrollPane actionScrollPane;
    JScrollPane planScrollPane;
   

    public ActionWindow(int width, int height, GanttChart c, GreedyPost gp, Grounding gi, String out) {
	super();
	
	//Plan info
	grounding = gi;
	greedyPost = gp;
	times = greedyPost.startTimes;
	durs = greedyPost.actDurs;
	plansize = greedyPost.planSize;
        plan = greedyPost.aPlan;
        actSigList = greedyPost.actSigList;
        order = greedyPost.orderingSet;
        output = out;

	//components
	chart = c;
	planDetailPanel = new JScrollPane(planDetail,
					  JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
					  JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	planDetailPanel.setPreferredSize(new Dimension(200, 
						       height-100));
	planDetailPanel.setVisible(true);
	add(planDetailPanel);

	actionScrollPane = new JScrollPane(actionDetail,
					   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
					   JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	actionScrollPane.setPreferredSize(new Dimension(200, 
							height-100));
	actionScrollPane.setVisible(true);
	add(actionScrollPane);

	setActionPanel();
	setPlanDetailPanel();

	tabPane = new JTabbedPane();
	tabPane.addTab("Actions", actionPanel);
	tabPane.addTab("Action Detail", actionScrollPane);
	tabPane.addTab("Plan Detail", planDetailPanel);
	add(tabPane);
    }

    public void newSize(int width, int height) {
	actionList.setFixedCellHeight(Math.max(chart.getStepHeight()-3,10));
	repaint();
    }

    public void setActionPanel() {
	actionPanel = new JPanel();
	actionPanel.setLayout(new BoxLayout(actionPanel, BoxLayout.Y_AXIS));
	cells = new Vector();
	String tmp_act_out ;
	for (int i = 0; i < plansize ; i++) {
	    tmp_act_out = ((String)actSigList.get(i));
	    tmp_act_out = tmp_act_out.substring(0, tmp_act_out.indexOf("*"));
	    
	    cells.add(i + " " + tmp_act_out);
	    
	}
	
	actionList = new JList(cells);
	actionList.setFixedCellHeight(Math.max(chart.getStepHeight()-3,10));
	actionList.setFixedCellWidth(200);
	
	actionList.addListSelectionListener(this);
	

	actionPanel.add(actionList);

    }

    public void setActionDetailPanel(int i) {
	if(actionDetail != null){
	    actionScrollPane.remove(actionDetail);	    
	}
	
	if(i > -1) {

	    if(actionList.getSelectedIndex() != i) {
		actionList.setSelectedIndex(i);
		
	    }

	    String tmp_act_out = ((String)actSigList.get(i));
	    String actName = tmp_act_out.substring(0, tmp_act_out.indexOf("*"));
	    
	    actionDetail = new JEditorPane();
	    String outString = 		 
		"<html><head></head><body>\n" +
		"Index: " + (i) + "\n" +
		"<p>Action: " + actName + "\n" +
		"<p>Start Time: " + times[i] + "\n" +
		"<p>End Time: " + (times[i] + durs[i]) + "\n" +
		"<p>Duration: " + durs[i] + "\n" +
		"<p>Objects:\n<ul>";
	    actionDetail.setEditable(false);
	    int last = tmp_act_out.indexOf("*");
	    int next_last;
	    while (last != -1) {
		next_last = tmp_act_out.indexOf("*", last + 1);
		if (next_last > 1)
		    outString += "<li>" + tmp_act_out.substring(last+1, next_last) + "\n";
		last = next_last;
	    }
	    outString += "</ul>";
	    
	    String causal = new String("");
	    String pmut = new String("");
	    String rmut = new String("");
	    int tmp;
	    
	    outString += "<p><font color=#00FF00>Pre Action Relations:</font> \n<ul>";
	    for(int j = 0; j < chart.pre_highlight.size(); j++){
		tmp = ((Integer)chart.pre_highlight.get(j)).intValue();
		tmp_act_out = ((String)actSigList.get(tmp));
		actName = tmp_act_out.substring(0, tmp_act_out.indexOf("*"));
		outString += "<li> " + actName + 
		    " (" + (String)chart.pre_highlight_rel.get(j) + ")\n";
	    }
	    
	    outString += "</ul><p><font color=#FF00FF>Post Action Relations:</font> \n<ul>";
	    for(int j = 0; j < chart.post_highlight.size(); j++){
		tmp = ((Integer)chart.post_highlight.get(j)).intValue();

		tmp_act_out = ((String)actSigList.get(tmp));
		actName = tmp_act_out.substring(0, tmp_act_out.indexOf("*"));
		outString += "<li>" + actName + 
		    " (" + (String)chart.post_highlight_rel.get(j) + ")\n";
		
	    }
	    outString += "</ul></body>\n</html>";
	    
	    try {
		actionDetail = new JEditorPane("text/html", outString);
		
	    }catch (Exception e){};
	    actionDetail.setVisible(true);
	    actionScrollPane.setViewportView(actionDetail);
	    this.repaint();
	}
    }
    
    public void unhighlight() {
	if(actionDetail != null) {
	    actionDetail.setVisible(false);
	    actionScrollPane.remove(actionDetail);
	    repaint();
	}
    }


    public void setPlanDetailPanel() {
	output += "</body></html>";
	try {
	    planDetail = new JEditorPane("text/html", output);
	    
	    planDetail.setEditable(false);

	}catch (Exception e){};
	planDetailPanel.setViewportView(planDetail);
    }
    

    public void setOptions(boolean[] options) {
	removeRedundant = options[4];
	repaint();
    }

    public void valueChanged(ListSelectionEvent e) {
	if(chart.highlight < 0 || (lastSelected != actionList.getSelectedIndex() && !chart.ganttHighlight)) {
	    chart.highlightCell(actionList.getSelectedIndex());
	    lastSelected = actionList.getSelectedIndex();
	}
	chart.ganttHighlight = false;	
    }
}
