/****************************************************************
     Author: Daniel Bryce - Arizona State University
****************************************************************/
package myplanner.gui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;


public class InputPane extends JPanel implements ActionListener{

    final JFileChooser fc = new JFileChooser();
    JButton domainButton;
    JTextField domainTextField;
    JButton problemButton;
    JTextField problemTextField;
    JButton startPlanButton;
    JCheckBox debugCB;
    JCheckBox costPropogationCB;
    JCheckBox relaxedPlanCB;
    JCheckBox goalCostAggCB;
    JCheckBox lookAheadCB;
    JTextField lookAheadTF;
    JCheckBox noAutoCB;
    JCheckBox qualityCB;
    JCheckBox relaxActionsCB;
    JCheckBox narelaxActionsCB;
    JCheckBox noResourceAdjustCB;
    JCheckBox timeLimitCB;
    JTextField timeLimitTF;
    JCheckBox cutoffFreqCB;
    JTextField cutoffFreqTF;
    JCheckBox hWtCB;
    JTextField hWtTF;
    JButton saveFileB;
    JTextField saveFileTF;

    JComboBox costPropogationOptionList;
    JComboBox goalCostAggList;
    String[] propogationOptions = {"max", "sum (default)", "Combo"};
    
    ArrayList args;
    boolean done;

    public InputPane() {
	args = new ArrayList(3);
	done = false;
        domainButton = new JButton("Choose Domain PDLL");
        domainTextField = new JTextField(30);

        problemButton = new JButton("Choose Problem FIle");
        problemTextField = new JTextField(30);

        startPlanButton = new JButton("Start Planning");

        debugCB = new JCheckBox("Debug");

        costPropogationCB = new JCheckBox("Cost Propogation");
	costPropogationCB.setSelected(true);
	costPropogationOptionList = new JComboBox(propogationOptions);
	costPropogationOptionList.setSelectedIndex(1);

	relaxedPlanCB = new JCheckBox("Don't compute Relaxed Plan");

	goalCostAggCB = new JCheckBox("Goal Cost Aggregation");
	goalCostAggCB.setSelected(true);
	goalCostAggList = new JComboBox(propogationOptions);
	goalCostAggList.setSelectedIndex(1);

	lookAheadCB = new JCheckBox("Lookahead");
	lookAheadCB.setSelected(true);
	lookAheadTF = new JTextField(10);
	lookAheadTF.setText("-1");

	noAutoCB =  new JCheckBox("No Auto");

	qualityCB = new JCheckBox("Quality");

	relaxActionsCB = new JCheckBox("Use helpful actions");
	narelaxActionsCB = new JCheckBox("Use negative effects of helpful actions");

	noResourceAdjustCB = new JCheckBox("Don't use resource adjustment technique");
	timeLimitCB = new JCheckBox("Timelimit (s)");
	timeLimitCB.setSelected(true);
	timeLimitTF = new JTextField(10);
	timeLimitTF.setText("7200");

	cutoffFreqCB = new JCheckBox("Frequency of cutoff check");
	cutoffFreqCB.setSelected(true);
	cutoffFreqTF = new JTextField(10);
	cutoffFreqTF.setText("200");

	hWtCB = new JCheckBox("Heuristic weight");
	hWtTF = new JTextField(10);

	saveFileB = new JButton("Choose Save File");
	saveFileTF = new JTextField(30);


	setLayout(new GridLayout(17,2));
	

        domainButton.addActionListener(this);
	problemButton.addActionListener(this);
	saveFileB.addActionListener(this);
	startPlanButton.addActionListener(this);


	add(domainButton);
	add(domainTextField);
	add(problemButton);
	add(problemTextField);
	add(debugCB);
	add(new JPanel());
	add(costPropogationCB);
	add(costPropogationOptionList);
	add(relaxedPlanCB);
	add(new JPanel());	
	add(goalCostAggCB);
	add(goalCostAggList);
	add(lookAheadCB);
	add(lookAheadTF);
	add(noAutoCB);
	add(new JPanel());
	add(qualityCB);
	add(new JPanel());
	add(relaxActionsCB);
	add(new JPanel());
	add(narelaxActionsCB);
	add(new JPanel());
	add(noResourceAdjustCB);
	add(new JPanel());
	add(timeLimitCB);
	add(timeLimitTF);
	add(cutoffFreqCB);
	add(cutoffFreqTF);
	add(hWtCB);
	add(hWtTF);
	add(saveFileB);
	add(saveFileTF);
       	add(startPlanButton);
    }

    public String[] getArgs() {
	if (!done)
	    return null;
	else {
	    String[] tmp = new String[args.size()];
	
	    for (int i = 0; i < args.size(); i++) 
		tmp[i] = (String)args.get(i);
		    
	    return (tmp);
	}
    }
    
    public void resetArgs() { args = new ArrayList(3); done = false; initArgs();}
    public void initArgs() {
	args.clear();
	if(domainTextField.getText().length() > 0)
	    args.add(0, domainTextField.getText());
	if(problemTextField.getText().length() > 0)
	    args.add(1, problemTextField.getText());	
    }

    public void actionPerformed(ActionEvent e) {
	JButton b = (JButton)e.getSource();
	if(b == domainButton) {
	    int returnVal = fc.showOpenDialog(InputPane.this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		if(args.size() >0)
		    args.remove(0);
		args.add(0, fc.getSelectedFile().getPath());
		domainTextField.setText((String)args.get(0));
	    }
	}
	else if (b == problemButton) {
	    int returnVal = fc.showOpenDialog(InputPane.this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		if(args.size() >1)
		    args.remove(1);
		args.add(1, fc.getSelectedFile().getPath());
		problemTextField.setText((String)args.get(1));
	    }
	}
	else if (b == saveFileB) {
	    int returnVal = fc.showSaveDialog(InputPane.this);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		saveFileTF.setText(fc.getSelectedFile().getPath());
	    }
	}
	else if (b == startPlanButton) {
	    
	    if(domainTextField.getText().length() > 0 && 
	       problemTextField.getText().length() > 0)
		done = true;
	    
	    
	    if(debugCB.isSelected())
		args.add(new String("-debug"));
	    
	    if (costPropogationCB.isSelected()) {
		
		String value = (String)costPropogationOptionList.getSelectedItem();
		args.add("-cp");
		if(value.equals("max"))
		    args.add("0");
		else if (value.equals("sum (default)"))
		    args.add("1");
		else if (value.equals("Combo"))
		    args.add("2");
		else
		    args.add("1");
		
	    }
	    
	    if(relaxedPlanCB.isSelected()) {
		args.add("-norp");
	    }
	    
	    if (goalCostAggCB.isSelected()) {
		
		String value = (String)goalCostAggList.getSelectedItem();
		args.add("-gc");
		if(value.equals("max"))
		    args.add("0");
		else if (value.equals("sum (default)"))
		    args.add("1");
		else if (value.equals("Combo"))
		    args.add("2");
		else
		    args.add("1");
		
	    }

	    if(lookAheadCB.isSelected()) {
		args.add("-la");
		if(lookAheadTF.getText().length() > 0)
		    args.add(lookAheadTF.getText());
		else
		    args.add("-1");
	    }
	    if(noAutoCB.isSelected()) {
		args.add("-noauto");
	    }
	    if(qualityCB.isSelected()) {
		args.add("-quality");
	    }
	    
	    if(relaxActionsCB.isSelected()) {
		args.add("-ha");
	    }
	    if(narelaxActionsCB.isSelected()) {
		args.add("-hane");
	    }
	    if(noResourceAdjustCB.isSelected()) {
		args.add("-noresadj");
	    }
	    
	    if(timeLimitCB.isSelected()) {
		args.add("-timelimit");
		if(timeLimitTF.getText().length() > 0)
		    args.add(timeLimitTF.getText());
		else
		    args.add("3600");
	    }
	    
	    if(cutoffFreqCB.isSelected()) {
		args.add("-freq");
		if(cutoffFreqTF.getText().length() > 0)
		    args.add(cutoffFreqTF.getText());
		else
		    args.add("200");
	    }
	    
	    if(hWtCB.isSelected()) {
		args.add("-hw");
		if(hWtTF.getText().length() > 0)
		    args.add(hWtTF.getText());
		else
		    args.add("200");
	    }
	    if(saveFileTF.getText().length()>0) {
		args.add("-outfile");
		args.add(saveFileTF.getText());

	    }
	    
	}
    }
}
