/************************************************************
   Author: Daniel Bryce -- Arizona State University
*************************************************************/
package myplanner.gui;

import myplanner.gui.GanttChart;
import myplanner.gui.ActionWindow;
import myplanner.gui.InputPane;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import myplanner.complex_ds.StateManager;
import myplanner.utils.GreedyPost;
import myplanner.parsing.Grounding;

public class SapaFrame extends JFrame implements ActionListener, ComponentListener, KeyListener, AdjustmentListener{
    int ROOT_WIDTH_DEFAULT = 800;
    int ROOT_HEIGHT_DEFAULT = 600;

    GanttChart chart;
    ActionWindow aw;
    public JPanel progressWindow;
    JTextPane progressText;
    Container contentPane;
    double zoomFactor = 0.1;
    JScrollPane scrollPane;
    JMenuBar menuBar;
    JMenu fileMenu;
    JMenu drawOptions;
    JMenuItem exitItem;
    JMenuItem newPlanItem;
    JFileChooser fileChooser;
    JCheckBoxMenuItem showCausalLinks;
    JCheckBoxMenuItem showPredMutex;
    JCheckBoxMenuItem showResMutex;
    JCheckBoxMenuItem showUnitActions;
    JCheckBoxMenuItem removeRedundantOrderings;
    boolean[] options = new boolean[5];
    InputPane inputPane;
    boolean done;
    String progressString;

    public SapaFrame (boolean lineArgs) {
	super("Sapa");
	setSize(ROOT_WIDTH_DEFAULT, ROOT_HEIGHT_DEFAULT);
	
	done = false;
	options[0] = true;
	options[1] = true;
	options[2] = true;
	options[3] = false;
	options[4] = true;

	menuBar = new JMenuBar();
	fileMenu = new JMenu("File");
	newPlanItem = new JMenuItem("New Plan");
	newPlanItem.setVisible(false);
	exitItem = new JMenuItem("Exit");

	newPlanItem.addActionListener(this);
	exitItem.addActionListener(this);
	

	setJMenuBar(menuBar);
	fileMenu.add(newPlanItem);
	fileMenu.addSeparator();
	fileMenu.add(exitItem);
	menuBar.add(fileMenu);


	contentPane = getContentPane();
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);}});
	addComponentListener(this);
	addKeyListener(this);
	drawOptions = new JMenu("Options");
	showCausalLinks = new JCheckBoxMenuItem("Causal Links",
						new ImageIcon("pic/red"));
	showPredMutex = new JCheckBoxMenuItem("Predicate Mutex", 
					      new ImageIcon("pic/blue"));
	showResMutex = new JCheckBoxMenuItem("Resource Mutex", 
					     new ImageIcon("pic/green"));
	showUnitActions = new JCheckBoxMenuItem("Unit Actions (Gantt/Pert)");
	removeRedundantOrderings = new JCheckBoxMenuItem("Remove Redundant Ordering Relations");
	
	showCausalLinks.addActionListener(this);
	showCausalLinks.setState(options[0]);
	showPredMutex.addActionListener(this);
	showPredMutex.setState(options[1]);
	showResMutex.addActionListener(this);
	showResMutex.setState(options[2]);
	showUnitActions.addActionListener(this);
	showUnitActions.setState(options[3]);
	removeRedundantOrderings.addActionListener(this);
	removeRedundantOrderings.setState(options[4]);
	
	drawOptions.add(showCausalLinks);
	drawOptions.add(showPredMutex);
	drawOptions.add(showResMutex);
	drawOptions.add(showUnitActions);
	drawOptions.add(removeRedundantOrderings);
	drawOptions.setVisible(false);
	menuBar.add(drawOptions);
	inputPane = new InputPane();
	
	setProgressWindow();
	setInputScreen(lineArgs);

	
    }

    void setProgressWindow() {
	progressString = new String("Searching...");
	progressWindow = new JPanel();
	progressWindow.setVisible(false);
	progressText = new JTextPane();
	progressText.setText(progressString);
	progressWindow.add(progressText);
	progressText.setBackground(Color.lightGray);
	contentPane.add(progressWindow, "Center");
    }

    public void showProgressWindow(boolean show) {
	progressWindow.setVisible(show);
	if(inputPane.isVisible())
	    inputPane.setVisible(!show);
    }


    private void setInputScreen(boolean lineArgs) {
	
	if(!lineArgs){
	    inputPane.setVisible(true);
	    contentPane.add(inputPane, "Center");
	}
	
    }

    public boolean done() { return done;}
    public String[] getArgs() {return inputPane.getArgs();}
    public void resetArgs() {inputPane.resetArgs();}
    public void showOutput(GreedyPost gp, Grounding gi, String output){
	newPlanItem.setVisible(true);
	drawOptions.setVisible(true);
	
	chart = new GanttChart(ROOT_WIDTH_DEFAULT,
			       ROOT_HEIGHT_DEFAULT,
			       gp,
			       gi,
			       options);
	aw = new ActionWindow(100,
			      ROOT_HEIGHT_DEFAULT,
			      chart,
			      gp,
			      gi,
			      output);
	chart.setAW(aw);
	scrollPane = new JScrollPane(chart,
				     JScrollPane.VERTICAL_SCROLLBAR_NEVER,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	scrollPane.setPreferredSize(new Dimension(ROOT_WIDTH_DEFAULT, 
						  ROOT_HEIGHT_DEFAULT+100));
	chart.addComponentListener(this);
	chart.addKeyListener(this);
	scrollPane.addComponentListener(this);
	scrollPane.getVerticalScrollBar().addAdjustmentListener(this);
	scrollPane.setVisible(true);
	inputPane.setVisible(false);
	contentPane.remove(inputPane);
	contentPane.add(aw, "West");
	contentPane.add(scrollPane, "Center");
	this.setVisible(true);
	

    }
    

/////////Events
    public void componentResized(ComponentEvent e) {
	Component c = e.getComponent();
	if(chart != null) {
	    chart.newSize(c.getWidth(), 
			  c.getHeight());
	    scrollPane.setPreferredSize(new Dimension(c.getWidth(), 
						      c.getHeight()));
	    aw.newSize(c.getWidth(), 
		       c.getHeight());
	}
	repaint();
    }
    public void componentMoved(ComponentEvent e) {}
    public void componentHidden(ComponentEvent e) {}
    public void componentShown(ComponentEvent e) {}
    public void componentAdded(ComponentEvent e) {}
    public void componentRemoved(ComponentEvent e) {}
    public void keyTyped(KeyEvent e) {
	int x;
	//int y;
	Component c = e.getComponent();
	char key = e.getKeyChar();
	System.out.println(key);
	if(key == 'q') System.exit(0);
	else if(key == '-') {
	    x = (int)(chart.getWidth()*(1.0 - zoomFactor));
	    //y = (int)(chart.getHeight()*(1.0 - zoomFactor));
	    if (chart.getWidth() > this.getWidth()-200)
		chart.newSize(x /*,y*/);
	}
	else if(key == '+') {
	    x = (int)(chart.getWidth()*(1.0 + zoomFactor));
	    //y = (int)(chart.getHeight()*(1.0 + zoomFactor));
	    chart.newSize(x/*,y*/);
	}
	repaint();
    }
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
    public void adjustmentValueChanged(AdjustmentEvent e) {repaint();}
    public void actionPerformed(ActionEvent e) {
	JMenuItem source = (JMenuItem)(e.getSource());
	if(source == exitItem) {
	    done = true;
	    System.exit(0);
	}
	else if(source == newPlanItem) {
	    newPlanItem.setVisible(false);
	    drawOptions.setVisible(false);
	    contentPane.remove(scrollPane);
	    contentPane.remove(aw);
	    contentPane.remove(chart);
	    repaint();
	    resetArgs();
	    setInputScreen(false);
	}
	else if(source == showCausalLinks){
	    options[0] = ((JCheckBoxMenuItem)source).getState();
	    chart.setDrawingOptions(options);
	}
	else if(source == showPredMutex){
	    options[1] = ((JCheckBoxMenuItem)source).getState();
	    chart.setDrawingOptions(options);
	}
	else if(source == showResMutex){
	    options[2] = ((JCheckBoxMenuItem)source).getState();
	    chart.setDrawingOptions(options);
	}
	else if(source == showUnitActions){
	    options[3] = ((JCheckBoxMenuItem)source).getState();
	    chart.setDrawingOptions(options);
	}
	else if(source == removeRedundantOrderings){
	    options[4] = ((JCheckBoxMenuItem)source).getState();
	    chart.setDrawingOptions(options);
	    aw.setOptions(options);
	}
    }
    /////////
}
